<?php

namespace Drupal\simple_survey\Entity;

use Drupal\Component\Datetime\Time;
use Drupal\Core\Config\Entity\ConfigEntityBase;
use Drupal\Core\Url;
use Drupal\simple_survey\Entity\SimpleSurveyInterface;

/**
 * Defines the SimpleSurvey entity.
 *
 * @ConfigEntityType(
 *   id = "simple_survey",
 *   label = @Translation("Simple feedback survey"),
 *   label_collection = @Translation("Simple feedback surveys"),
 *   label_singular = @Translation("Simple feedback survey"),
 *   label_plural = @Translation("Simple feedback surveys"),
 *   label_count = @PluralTranslation(
 *     singular = "@count Simple feedback survey",
 *     plural = "@count Simple feedback surveys",
 *   ),
 *   handlers = {
 *     "list_builder" = "Drupal\simple_survey\Controller\SimpleSurveyListBuilder",
 *     "form" = {
 *       "add" = "Drupal\simple_survey\Form\SimpleSurveyForm",
 *       "edit" = "Drupal\simple_survey\Form\SimpleSurveyForm",
 *       "delete" = "Drupal\simple_survey\Form\SimpleSurveyDeleteForm"
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider",
 *     },
 *   },
 *   config_prefix = "simple_survey",
 *   admin_permission = "administer simple survey configuration",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label"
 *   },
 *   config_export = {
 *     "id",
 *     "label",
 *     "debug",
 *     "description",
 *     "basePath",
 *     "questionsPath",
 *     "submitPath",
 *     "validationPath",
 *     "enableTranslation",
 *     "anchorTag",
 *     "attachStylesheet",
 *     "displayDelayType",
 *     "delay",
 *     "delayPosition",
 *     "displayOnceOnSite",
 *     "closingDelay",
 *     "bigQueryProjectId",
 *     "bigQueryKeyfile",
 *     "bigQueryTable",
 *     "questionWeights",
 *     "useFontAwesome",
 *     "displayFilterByPath",
 *     "startingQuestion",
 *     "enableSurvey",
 *     "numberOfResponsesToStopAt",
 *     "surveyActiveFrom",
 *     "surveyActiveUntil",
 *     "ombDisclaimer",
 *     "ombId",
 *     "ombExpiration",
 *     "ombLink"
 *   },
 *   links = {
 *     "canonical" = "/admin/content/simple_survey/{simple_survey}",
 *     "add-form" = "/admin/content/simple_survey/add",
 *     "edit-form" = "/admin/content/simple_survey/{simple_survey}/edit",
 *     "delete-form" = "/admin/content/simple_survey/{simple_survey}/delete",
 *     "collection" = "/admin/content/simple_survey",
 *   }
 * )
 */
class SimpleSurvey extends ConfigEntityBase implements SimpleSurveyInterface {

  /**
   * Survey in debug mode.
   *
   * @var bool
   */
  public $debug;

  /**
   * Survey Base Path.
   *
   * @var string
   */
  public $basePath;

  /**
   * Survey Question Path.
   *
   * @var string
   */
  public $questionsPath;

  /**
   * Survey Submission Path.
   *
   * @var string
   */
  public $submitPath;

  /**
   * Survey validation Path.
   *
   * @var string
   */
  public $validationPath;

  /**
   * Enable Translation.
   *
   * @var bool
   */
  public $enableTranslation;

  /**
   * The SimpleSurvey ID.
   *
   * @var string
   */
  public $id;

  /**
   * The SimpleSurvey label.
   *
   * @var string
   */
  public $label;

  /**
   * A brief description of this simple feedback survey.
   *
   * @var string
   */
  public $description;

  /**
   * HTML tag or ID Survey should anchor to.
   *
   * @var string
   */
  public $anchorTag;

  /**
   * Attach module stylesheet.
   *
   * @var bool
   */
  public $attachStylesheet;

  /**
   * How should the survey display.
   *
   * @var string
   */
  public $displayDelayType;

  /**
   * How long before the survey should appear.
   *
   * @var int
   */
  public $delay;

  /**
   * Only display once on the entire site.
   *
   * @var bool
   */
  public $displayOnceOnSite;

  /**
   * Amount of time before survey closes on last screen.
   *
   * @var int
   */
  public $closingDelay;

  /**
   * If using positional delay where should it appear.
   *
   * @var string
   */
  public $delayPosition;

  /**
   * Use font awesome if it is available.
   *
   * @var bool
   */
  public $useFontAwesome;

  /**
   * Project ID in BigQuery.
   *
   * @var string
   */
  public $bigQueryProjectId;

  /**
   * BigQuery key file text.
   *
   * @var string
   */
  public $bigQueryKeyfile;

  /**
   * Table used in BigQuery.
   *
   * @var bool
   */
  public $bigQueryTable;

  /**
   * Question weights.
   *
   * @var array
   */
  public $questionWeights;

  /**
   * Display page filtering.
   *
   * @var array
   */
  public $displayFilter;

  /**
   * Display page filtering path.
   *
   * @var array
   */
  public $displayFilterByPath;

  /**
   * The starting Question.
   *
   * @var array
   */
  public $startingQuestion;

  /**
   * Display page filtering.
   *
   * @var int
   */
  public $displayCooldown;

  /**
   * Display page filtering path.
   *
   * @var int
   */
  public $answeredDisplayCooldown;

  /**
   * Enables survey.
   *
   * @var bool
   */
  public $enableSurvey;

  /**
   * Date survey should be active from.
   *
   * @var string
   */
  public $surveyActiveFrom;

  /**
   * Date survey should be active to.
   *
   * @var string
   */
  public $surveyActiveUntil;

  /**
   * If the form should display and Privacy Disclaimer.
   *
   * @var bool
   */
  public $ombDisclaimer;

  /**
   * Privacy document ID if any.
   *
   * @var string
   */
  public $ombId;

  /**
   * Expiration of the document.
   *
   * @var string
   */
  public $ombExpiration;

  /**
   * Link to privacy document.
   *
   * @var string
   */
  public $ombLink;

  /**
   * Number of responses survey should stop collecting data at.
   *
   * @var int
   */
  public $numberOfResponsesToStopAt;

  /**
   * Returns the debug property.
   *
   * @return bool
   *   The debug property value.
   */
  public function debug() {
    return $this->debug;
  }

  /**
   * Returns the anchorTag property.
   *
   * @return string
   *   The anchorTag property value.
   */
  public function anchorTag() {
    return $this->anchorTag;
  }

  /**
   * {@inheritdoc}
   */
  public function description() {
    return $this->description;
  }

  /**
   * {@inheritdoc}
   */
  public function getDebug() {
    return $this->debug;
  }

  /**
   * Checks if the entered paths match the current path.
   *
   * @param string $path
   *   The current aliased path.
   *
   * @return bool
   *   Returns true if a match is found false if not.
   */
  public function checkPath(string $path) {
    $pathArray = preg_split("/\r\n|\n/", $this->displayFilterByPath);

    foreach ($pathArray as $testPath) {
      if ($testPath == '<front>' && \Drupal::service('path.matcher')->isFrontPage()) {
        return TRUE;
      }

      if (preg_match('/^' . preg_quote($testPath, '/') . '/', $path)) {
        return TRUE;
      }
    }

    return FALSE;
  }

  /**
   * Get the survey that loads on this path.
   *
   * @return null|\Drupal\simple_survey\Entity\SimpleSurveyInterface
   *   Returns either null or the Simple Survey entity.
   */
  public static function getSurvey() {
    $currentUrl = Url::fromRoute('<current>')->toString();

    $surveys = self::loadMultiple();

    foreach ($surveys as $survey) {
      if ($survey) {
        if (!$survey->get('enableSurvey')) {
          continue;
        }

        if (!$survey->hasClosingQuestion() || !$survey->get('startingQuestion')) {
          continue;
        }

        if ($survey->checkPath($currentUrl)) {
          return $survey->reachedMaximumResponses() || $survey->reachedSurveyExpiration()
            ? NULL : $survey;
        }
      }
    }

    return NULL;
  }

  /**
   * Checks survey not expired.
   */
  public function reachedSurveyExpiration() {
    $endDate = !empty($this->get('surveyActiveUntil')) ?
      strtotime($this->get('surveyActiveUntil')) + 86400 : NULL;
    $currentDate = \Drupal::time()->getRequestTime();

    return $endDate ? $endDate <= $currentDate : FALSE;
  }

  /**
   * Gets the maximum number of responses.
   */
  public function reachedMaximumResponses() {
    $numberOfResponses = \Drupal::service('entity_type.manager')->getStorage('survey_response')->getQuery()
      ->condition('survey', $this->id())
      ->count()
      ->execute();

    $maxNumberOfResponses = $this->get('numberOfResponsesToStopAt') ?: 0;

    return $maxNumberOfResponses > 0 ? $numberOfResponses >= $maxNumberOfResponses : FALSE;
  }

  /**
   * Checks if this survey has a closing question.
   *
   * @return bool
   *   Returns true if the count query returns greater than zero.
   */
  public function hasClosingQuestion() {
    return \Drupal::service('entity_type.manager')->getStorage('survey_question')->getQuery()
      ->condition('survey', $this->id())
      ->condition('question_type', 'closing')
      ->count()
      ->execute() > 0;
  }

}
